/* Only this file, Makefile, and cfg-mplayer.h should be modified to add
   support for new Bossa policies */

/* ----------------------- global variables ---------------------- */

int in_bossa = 0; // 1 if in Bossa
char *policy = "";

/* ------------------------- entry points ------------------------ */

void (*end_bossa)(void);
void (*pause_bossa)(void);

/* --------------- variables used by mplayer flags --------------- */

// variables for EDF and EDFu
int wcet = 0;
int friend = 0;

/* ------------------------- policy code ------------------------- */

#ifdef BOSSA_EDF
#include "user_stub_EDF.h"
#include "user_stub_Fixed_priority.h"
#include "user_stub_Linux.h"
int period = 0;

void edf_start_bossa(void) {
  int prop;
  if (period && wcet) {
    prop = 10;
    mount_root("Fixed_priority",1,&prop);
    prop = 20;
    mount("Fixed_priority","EDF",1,&prop);
    EDF_attach(0,wcet,period);
    if (friend) Linux_sched_setscheduler(friend,BOSSA_SCHED_RR,20);
  }
  else {
    printf("error - must specify both period and wcet\n");
    printf("period %d wcet %d\n",period,wcet);
    exit(1);
  }
}

void edf_end_bossa(void) {
  Linux_attach(0,BOSSA_SCHED_OTHER,0,20);
  unmount("EDF");
  unmount("Fixed_priority");
  if (friend) Linux_sched_setscheduler(friend,BOSSA_SCHED_OTHER,0);
  in_bossa = 0;
}

void edf_pause_bossa(void) {
  EDF_sched_yield(0);
}
#endif

#ifdef BOSSA_EDFU
#include "user_stub_EDFu.h"
#include "user_stub_Fixed_priority.h"
#include "user_stub_Linux.h"

void edfu_start_bossa(void) {
  int prop;
  if (wcet) {
    prop = 10;
    mount_root("Fixed_priority",1,&prop);
    prop = 20;
    mount("Fixed_priority","EDFu",1,&prop);
    if (EDFu_attach(0,wcet,(int)(sh_video->fps)) < 0) exit(0);
    if (friend) Linux_sched_setscheduler(friend,BOSSA_SCHED_RR,20);
  }
  else {
    printf("error - must specify both frames per second and wcet\n");
    printf("frames per second %d wcet %d\n",sh_video->fps,wcet);
    exit(1);
  }
}

void edfu_end_bossa(void) {
  Linux_attach(0,BOSSA_SCHED_OTHER,0,20);
  unmount("EDFu");
  unmount("Fixed_priority");
  if (friend) Linux_sched_setscheduler(friend,BOSSA_SCHED_OTHER,0);
  in_bossa = 0;
}

void edfu_pause_bossa(void) {
  EDFu_sched_yield(0);
}
#endif

#include "user_stub_Linux.h"

void linux_start_bossa(void) {
  if (friend) Linux_sched_setscheduler(friend,BOSSA_SCHED_RR,20);
  in_bossa = 0;
}

void linux_end_bossa(void) {
  printf("should never run!!!\n");
  exit(0);
}

void linux_pause_bossa(void) {
  printf("should never run!!!\n");
  exit(0);
}

/* ----------------------- initialization ------------------------ */

void start_bossa(void) {
  in_bossa = 1;
  if (strcmp(policy,"EDF") == 0) {
    end_bossa = edf_end_bossa;
    pause_bossa = edf_pause_bossa;
    edf_start_bossa();
  }
  else if (strcmp(policy,"EDFu") == 0) {
    end_bossa = edfu_end_bossa;
    pause_bossa = edfu_pause_bossa;
    edfu_start_bossa();
  }
  else if (strcmp(policy,"Linux") == 0) {
    end_bossa = linux_end_bossa;
    pause_bossa = linux_pause_bossa;
    linux_start_bossa();
  }
  else {
    in_bossa = 0;
  }
}
